const WP = require('../../lib/wxParse/wxParse.js')
const { getLotteryDetail, drawLottery, invite } = require('../../utils/API.js')
const USER_INFO = wx.getStorageSync('USER_INFO')
const NOTICE = `Lucky Maker作为提供抽奖服务的平台, 不对第三方发起人行为承担任何责任。活动奖品均由发起人提供并派发，Lucky Maker会在法律范围内尽可能地规范实行抽奖活动，感谢支持。`

Page({
	data: {
		title: '',
		id: '',
		winner_count: 0, // 奖品名额
		swiperIndex: 0,
		cover_images: [], // 缩略图
		draw_ts: '', //开奖时间
		host_avatar_url: '', // 发起人头像
    host_name: '', // 发起人姓名
    host_detail: '', // 发起人信息
    host_type: '1', // 发起人类型   1: 公众号  2： 小程序   3： 其他
		state: '0', // 奖项抽奖状态     0: 未抽奖  1: 已抽奖   2: 已结束
    winner_codes: [], // 中奖码
    download_url:'',  // 下载链接
		invited_user_avatars: [], //  我邀请的队友
    lottery_user_count: 0, // 抽奖参与总人数
    lottery_user_avatars: [], // 参与抽奖的用户头像
		user_lottery_codes: [], //  用户的抽奖码
    recommend_lottery: {}, //  推荐的抽奖
    visit_count: 0, // 浏览数
		user_draw_state: 0, // 用户抽奖状态
    isFold: false, // 当前内容折叠状态
    needFold: false, // 当前内容高度是否需要折叠
		modalVisiable: false,
    notice: NOTICE,
    shareInfo: null
	},
	swipeTo(ev) {
		const { index } = ev.target.dataset
		this.setData({ swiperIndex: index })
	},
	hideModal() {
		this.setData({ modalVisiable: false })
	},
	showModal() {
		this.setData({ modalVisiable: true })
  },
  // 切换收起折叠
	toggleFold() {
		this.setData({ isFold: !this.data.isFold })
  },
  // 根据抽奖内容多少判断是否需要折叠
  _judgeContentHeight(){
    wx.createSelectorQuery().select('.award-article').boundingClientRect(nodeRef => {
      nodeRef.height > 1000 ? this.setData({needFold: true}) : this.setData({needFold: false})
    }).exec()
  },
  // 点击抽奖按钮事件处理函数
	handleDrawLottery() {
    const { id } = this.data
    wx.showLoading({
      title: '抽奖中'
    });
		drawLottery(id)
			.then(data => {
        const {user_lottery_codes, avatar_url} = data.data
        wx.hideLoading()
				wx.showToast({ title: data.msg })
				this.setData({
          user_draw_state: 1,
          user_lottery_codes: this.data.state === 2 ? user_lottery_codes.map(code => ({code, isWin: winner_codes.includes(code)})) : user_lottery_codes.map(code => ({code, isWin: false})),
          lottery_user_avatars: this.data.lottery_user_avatars.concat(avatar_url)
        })
        // 抽奖成功后更新一下列表页的视图
        const prevPage = getCurrentPages().shift()
        let _index = prevPage.data.lotteries.findIndex(item => item.id === id)
        let key = `lotteries[${_index}].user_draw_state`
        prevPage.setData({
          [key]: 1
        })
			})
			.catch(err => {
				wx.showToast({ title: err.msg })
			})
  },
  // 点击邀请按钮事件处理函数
	onShareAppMessage() {
    const { id, cover_images, title, winner_count } = this.data
    return {
      title: '抽奖：' + title + ' X ' + winner_count,
      imageUrl: cover_images[0],
      path: '/pages/award-detail/award-detail?id=' + id + '&from=share&friendID=' + wx.getStorageSync('USER_INFO') ? wx.getStorageSync('USER_INFO').user.id : null
    }
  },
	onLoad(query) {
    wx.showLoading({title: '加载中', mask: true})
    if(!USER_INFO) {
      wx.navigateTo({url: '/pages/login/login'})
    }
    const { id } = query
    // 当用户是从分享页面进入时调用邀请接口
    if(query.from && query.from === 'share' && query.friendID) {
      if(query.friendID !== USER_INFO.user.id) {
        invite(id, query.friendID)
        .then(data => {
          console.log(data);
        })
        .catch(err => {
          console.log(err);
        })
      }
    }
		getLotteryDetail(id).then(data => {
			const {
				body,
				cover_images,
				title,
				draw_ts,
				winner_count,
				host_avatar_url,
				host_name,
				state,
				winner_codes,
				invited_user_avatars,
				lottery_user_count,
				user_lottery_codes,
        recommend_lottery,
        user_draw_state,
        id,
        host_detail,
        visit_count,
        download_url,
        lottery_user_avatars
      } = data.data
			this.setData({
				cover_images,
				title,
				winner_count,
				draw_ts,
				host_avatar_url,
				host_name,
				winner_codes,
				state,
				invited_user_avatars: invited_user_avatars.slice(0, 24),
        lottery_user_count,
        // 抽奖结束时判断用户是否中奖
				user_lottery_codes: state === '2' ? user_lottery_codes.map(code => ({code, isWin: winner_codes.includes(code)})) : user_lottery_codes.map(code => ({code, isWin: false})),
        recommend_lottery,
        user_draw_state,
        id,
        host_detail,
        visit_count,
        download_url,
        lottery_user_avatars,
        shareInfo: {
          id,
          title: title,
          count: winner_count,
          imageUrl: cover_images[0],
          time: state === '2' ? '已结束' : draw_ts + ' 自动开奖',
          scene: 'id=' + id + '&from=share&friendID=' + wx.getStorageSync('USER_INFO').user.id,
          path: 'pages/award-detail/award-detail'
        }
			})
      WP.wxParse('article', 'html', body, this, 5)
      this._judgeContentHeight()
      wx.hideLoading()
		})
	},
	seeAll() {
		wx.navigateTo({
			url: '/pages/allParticipant/allParticipant?id=' + this.data.id
		})
  },
  download(){
    const { download_url } = this.data
    wx.showModal({
      title: '点击复制下载链接',
      content: download_url,
      confirmText: '复制',
      success(res){
        if(res.confirm) {
          wx.setClipboardData({
            data: download_url,
            success(){
              wx.showToast({title: '复制成功'})
            }
          })

        }
      }
    })
  },
  test(){
    wx.navigateTo({url: '/pages/article/article'})
  }
})
